try:
    from functools import wraps
except ImportError:
    def wraps(func):
        def _wrapper(decorator):
            decorator.__doc__ = func.__doc__
            decorator.__dict__ = func.__dict__
            decorator.__name__ = func.__name__
            
            return decorator
        return _wrapper
    
try:
    import hashlib
    _md5 = hashlib.md5
except ImportError:
    import md5
    _md5 = md5.new

from django.conf import settings
from django.core.cache import cache

def cached(func):
    @wraps(func)
    def _decorator(filename):
        name = settings.COAT_CACHE_PREFIX + _md5(filename).hexdigest()
        value = cache.get(name)
        if not value:
            value = func(filename)
            cache.set(name, value, settings.COAT_CACHE_TIME)
        return value
    return _decorator
